@@ -11,6 +11,7 @@ from espider.settings import Setting
 from espider.utils import requests
 from espider.utils.tools import url_to_dict, body_to_dict, json_to_dict, headers_to_dict, cookies_to_dict, dict_to_body, \
     dict_to_json, update, search, delete, strip, replace, random_list, human_time
+from selenium import webdriver
 
 
 class Spider(object):
@@ -190,15 +191,15 @@ class Spider(object):
         if hasattr(response, 'cookies'):
             self.spider['cookies'] = merge_cookies(self.spider.get('cookies'), response.cookies)
 
+    @staticmethod
+    def chrome_driver(options=None):
+        return webdriver.Chrome(options=options)
+
     def request(self, url=None, method=None, data=None, json=None, headers=None, cookies=None, callback=None,
                 args=None, priority=None, use_session=None, **kwargs):
 
         if use_session is None: use_session = self.use_session
 
-        if isinstance(headers, str): headers = headers_to_dict(headers)
-        if isinstance(cookies, str): cookies = cookies_to_dict(cookies)
-        if isinstance(json, str): json = json_to_dict(json)
-
         # 请求优先级自增
         if not priority:
             pre_func_name = traceback.extract_stack()[-2].name
